/*------------------------------------------------------------------------------
* sdr.h: sdr constants, types and function prototypes
*
*          Copyright (C) 2010 by T.TAKASU, All rights reserved.
*
* options : -DSSE2   use sse2 instructions
*           -DFFTW   use fftw
*           -DMKLFFT use mkl dft with fftw interface
*           -DCUFFT  use cufft with cuda
*
* version : $Revision:$ $Date:$
* history : 2010/10/31 1.0  new
*-----------------------------------------------------------------------------*/
#ifndef SDR_H
#define SDR_H
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#if defined(SSE2)
#include <emmintrin.h>
#endif

#if defined (FFTW)
#include "fftw3.h"
#elif defined (MKLFFT)
#include "fftw.h"
#include "fftw2_mkl.h"
#elif defined (CUFFT)
#include "cufft.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif
/* constants -----------------------------------------------------------------*/

#define PI          3.1415926535897932  /* pi */
#define DPI         (2.0*PI)            /* 2*pi */

/* type definition -----------------------------------------------------------*/
#if defined(FFTW)||(MKLFFT)
typedef fftwf_complex cpx_t;            /* complex type for fft functions */
#elif defined(CUFFT)
typedef cufftComplex cpx_t;
#endif

/* sdr receiver fft functions ------------------------------------------------*/
extern cpx_t *cpxmalloc(int n);
extern void cpxfree(cpx_t *cpx);
extern void cpxcpx(const short *real, const short *img, double scale, int n,
                   cpx_t *cpx);
extern void cpxfft(cpx_t *cpx, int n);
extern void cpxifft(cpx_t *cpx, int n);
extern void cpxmul(const cpx_t *cpxa, const cpx_t *cpxb, int flag, int n);
extern void cpxabs(const cpx_t *cpx, int n, double *s);

/* sdr receiver code generator -----------------------------------------------*/
extern void gencode(int sat, int ctype, double coff, double ci, int n,
                    short *code);

/* sdr receiver correlator ---------------------------------------------------*/
extern void corrfunc(const char *data, int dtype, double ti, int n,
                     double freq, int sat, int ctype, double crate, double *P);

extern void correlator(const char *data, int dtype, double ti, int n,
                       double freq, double phi0, int sat, int ctype,
                       double crate, double coff, int s, int m, double *I,
                       double *Q);

#ifdef __cplusplus
}
#endif
#endif /* SDR_H */
